@@ -1,4 +1,4 @@
-import gpx
+import gcodex3g as gpx
 gpx.connect("/dev/ttyACM0", 0, "/home/pi/gpx.ini")
 print gpx.write("M105")
 print gpx.write("M114")
